# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "읽기 %d분 남음.",
    "(active)": "(활성됨)",
    "Also available in:": "이곳에서도 가능함:",
    "Archive": "저장소",
    "Atom feed": "아톰 피드",
    "Authors": "작성자",
    "Categories": "분류",
    "Comments": "댓글",
    "LANGUAGE": "한국어",
    "Languages:": "언어:",
    "More posts about %s": "%s에 대한 또다른 포스트",
    "Newer posts": "최신 포스트",
    "Next post": "다음 포스트",
    "Next": "다음",
    "No posts found.": "검색된 포스트 없음.",
    "Nothing found.": "검색 결과 없음.",
    "Older posts": "옛날 포스트",
    "Original site": "출처",
    "Posted:": "작성됨:",
    "Posts about %s": "%s에 대한 포스트",
    "Posts by %s": "%s의 작성된 포스트",
    "Posts for year %s": "%s년도 포스트",
    "Posts for {month_day_year}": "{month_day_year}에 작성된 포스트",
    "Posts for {month_year}": "{month_year}에 쓴 포스트",
    "Previous post": "이전 포스트",
    "Previous": "이전",
    "Publication date": "발간일",
    "RSS feed": "RSS 목록",
    "Read in English": "영어로 읽기",
    "Read more": "더 읽기",
    "Skip to main content": "주 콘텐츠로 바로가기",
    "Source": "원문",
    "Subcategories:": "서브 카테고리:",
    "Tags and Categories": "태그와 분류",
    "Tags": "태그",
    "Toggle navigation": "네비게이션 켜고 끄기",
    "Uncategorized": "카테고리 없음",
    "Up": "상위",
    "Updates": "업데이트",
    "Write your page here.": "여기에 페이지를 작성하세요.",
    "Write your post here.": "이곳에 글을 작성하세요.",
    "old posts, page %d": "이전 포스트, 페이지 %d",
    "page %d": "페이지 %d",
    "updated": "업데이트됨",
}



"""Check that per-day archives build and are correct."""

import os

import pytest

from nikola import __main__

from .helper import cd, patch_config
from .test_demo_build import prepare_demo_site
from .test_empty_build import (  # NOQA
    test_archive_exists,
    test_avoid_double_slash_in_rss,
    test_check_files,
    test_check_links,
    test_index_in_sitemap,
)


def test_day_archive(build, output_dir):
    """See that it builds"""
    archive = os.path.join(output_dir, "2012", "03", "30", "index.html")
    assert os.path.isfile(archive)


@pytest.fixture(scope="module")
def build(target_dir):
    """Fill the site with demo content and build it."""
    prepare_demo_site(target_dir)

    patch_config(
        target_dir, ("# CREATE_DAILY_ARCHIVE = False", "CREATE_DAILY_ARCHIVE = True")
    )

    with cd(target_dir):
        __main__.main(["build"])



"""
Test a site with translated content.

Do not test titles as we remove the translation.
"""

import io
import os
import shutil

import lxml.html
import pytest

import nikola.plugins.command.init
from nikola import __main__

from .helper import cd
from .test_empty_build import (  # NOQA
    test_archive_exists,
    test_avoid_double_slash_in_rss,
    test_check_files,
    test_check_links,
    test_index_in_sitemap,
)


def test_translated_titles(build, output_dir, other_locale):
    """Check that translated title is picked up."""
    normal_file = os.path.join(output_dir, "pages", "1", "index.html")
    translated_file = os.path.join(output_dir, other_locale, "pages", "1", "index.html")

    # Files should be created
    assert os.path.isfile(normal_file)
    assert os.path.isfile(translated_file)

    # And now let's check the titles
    with io.open(normal_file, "r", encoding="utf8") as inf:
        doc = lxml.html.parse(inf)
        assert doc.find("//title").text == "Foo | Demo Site"

    with io.open(translated_file, "r", encoding="utf8") as inf:
        doc = lxml.html.parse(inf)
        assert doc.find("//title").text == "Bar | Demo Site"


@pytest.fixture(scope="module")
def build(target_dir, test_dir):
    """Build the site."""
    init_command = nikola.plugins.command.init.CommandInit()
    init_command.create_empty_site(target_dir)
    init_command.create_configuration(target_dir)

    src = os.path.join(test_dir, "..", "data", "translated_titles")
    for root, dirs, files in os.walk(src):
        for src_name in files:
            rel_dir = os.path.relpath(root, src)
            dst_file = os.path.join(target_dir, rel_dir, src_name)
            src_file = os.path.join(root, src_name)
            shutil.copy2(src_file, dst_file)

    with cd(target_dir):
        __main__.main(["build"])



import asyncio
import nikola.plugins.command.auto as auto
from nikola.utils import get_logger
import pytest
import pathlib
import requests
import socket
from typing import Optional, Tuple, Any, Dict

from ..helper import FakeSite

SERVER_ADDRESS = "localhost"
TEST_MAX_DURATION = 10  # Watchdog: Give up the test if it did not succeed during this time span.

# Folder that has the fixture file we expect the server to serve:
OUTPUT_FOLDER = pathlib.Path(__file__).parent.parent / "data" / "dev_server_sample_output_folder"

LOGGER = get_logger("test_dev_server")


def find_unused_port() -> int:
    """Ask the OS for a currently unused port number.

    (More precisely, a port that can be used for a TCP server servicing SERVER_ADDRESS.)
    We use a method here rather than a fixture to minimize side effects of failing tests.
    """
    s = socket.socket()
    try:
        ANY_PORT = 0
        s.bind((SERVER_ADDRESS, ANY_PORT))
        address, port = s.getsockname()
        LOGGER.info("Trying to set up dev server on http://%s:%i/", address, port)
        return port
    finally:
        s.close()


class MyFakeSite(FakeSite):
    def __init__(self, config: Dict[str, Any], configuration_filename="conf.py"):
        super(MyFakeSite, self).__init__()
        self.configured = True
        self.debug = True
        self.THEMES = []
        self._plugin_places = []
        self.registered_auto_watched_folders = set()
        self.config = config
        self.configuration_filename = configuration_filename


def test_serves_root_dir(
    site_and_base_path: Tuple[MyFakeSite, str], expected_text: str
) -> None:
    site, base_path = site_and_base_path
    command_auto = auto.CommandAuto()
    command_auto.set_site(site)
    options = {
        "browser": False,
        "ipv6": False,
        "address": SERVER_ADDRESS,
        "port": find_unused_port(),
        "db-file": "/dev/null",
        "backend": "No backend",
        "no-server": False,
        "poll": False
    }

    # We start an event loop, run the test in an executor,
    # and wait for the event loop to terminate.
    # These variables help to transport the test result to
    # the main thread outside the event loop:
    test_was_successful = False
    test_problem_description = "Async test setup apparently broken"
    test_inner_error: Optional[BaseException] = None
    loop = None

    async def grab_loop_and_run_test() -> None:
        nonlocal test_problem_description, loop

        loop = asyncio.get_running_loop()
        watchdog_handle = loop.call_later(TEST_MAX_DURATION, loop.stop)
        test_problem_description = f"Test did not complete within {TEST_MAX_DURATION} seconds."

        def run_test() -> None:
            nonlocal test_was_successful, test_problem_description, test_inner_error
            try:
                with requests.Session() as session:
                    server_root_uri = f"http://{options['address']}:{options['port']}"

                    # First subtest: Grab the document root index.html file:
                    server_base_uri = f"{server_root_uri}{base_path}"
                    LOGGER.info("Attempting to fetch HTML from %s", server_base_uri)
                    res = session.get(server_base_uri)
                    res.raise_for_status()
                    assert "text/html; charset=utf-8" == res.headers['content-type']
                    assert expected_text in res.text

                    # Second subtest: Does the dev server serve something for the livereload JS?
                    js_uri = f"{server_root_uri}/livereload.js?snipver=1"
                    LOGGER.info("Attempting to fetch JS from %s", js_uri)
                    res_js = session.get(js_uri)
                    res_js.raise_for_status()
                    content_type_js = res_js.headers['content-type']
                    assert "javascript" in content_type_js

                test_was_successful = True
                test_problem_description = "No problem. All is well."
            except BaseException as be:
                LOGGER.error("Could not receive HTTP as expected.", exc_info=True)
                test_inner_error = be
                test_was_successful = False
                test_problem_description = "(see exception)"
            finally:
                if test_was_successful:
                    LOGGER.info("Test completed successfully.")
                else:
                    LOGGER.error("Test failed: %s", test_problem_description)
                loop.call_soon_threadsafe(watchdog_handle.cancel)
                # Simulate Ctrl+C:
                loop.call_soon_threadsafe(lambda: loop.call_later(0.01, loop.stop))

        await loop.run_in_executor(None, run_test)

    # We defeat the nikola site building functionality, so this does not actually get called.
    # But the code setting up site building wants a command list:
    command_auto.nikola_cmd = ["echo"]

    # Defeat the site building functionality, and instead insert the test:
    command_auto.run_initial_rebuild = grab_loop_and_run_test

    # Start the development server
    # which under the hood runs our test when trying to build the site:
    command_auto.execute(options=options)

    # Verify the test succeeded:
    if test_inner_error is not None:
        raise test_inner_error
    assert test_was_successful, test_problem_description


@pytest.fixture(scope="module",
                params=["https://example.org",
                        "https://example.org:1234/blog",
                        "https://example.org:3456/blog/",
                        "http://example.org/deep/down/a/rabbit/hole"
                        ])
def site_and_base_path(request) -> Tuple[MyFakeSite, str]:
    """Return a fake site and the base_path (root) the dev server should be serving."""
    assert OUTPUT_FOLDER.is_dir(), \
        f"Could not find dev server test fixture {OUTPUT_FOLDER.as_posix()}"

    config = {
        "post_pages": [],
        "FILES_FOLDERS": [],
        "GALLERY_FOLDERS": [],
        "LISTINGS_FOLDERS": [],
        "IMAGE_FOLDERS": [],
        "SITE_URL": request.param,
        "OUTPUT_FOLDER": OUTPUT_FOLDER.as_posix(),
    }
    return MyFakeSite(config), auto.base_path_from_siteuri(request.param)


@pytest.fixture(scope="module")
def expected_text():
    """Read the index.html file from the fixture folder and return most of it.

    For life reload, the server will fiddle with HTML <head>,
    so this only returns everything after the opening <body> tag.
    """
    with open(OUTPUT_FOLDER / "index.html", encoding="utf-8") as html_file:
        all_html = html_file.read()
        return all_html[all_html.find("<body>"):]


@pytest.mark.parametrize(("uri", "expected_basepath"), [
    ("http://localhost", ""),
    ("http://local.host", ""),
    ("http://localhost/", ""),
    ("http://local.host/", ""),
    ("http://localhost:123/", ""),
    ("http://local.host:456/", ""),
    ("https://localhost", ""),
    ("https://local.host", ""),
    ("https://localhost/", ""),
    ("https://local.host/", ""),
    ("https://localhost:123/", ""),
    ("https://local.host:456/", ""),
    ("http://example.org/blog", "/blog"),
    ("https://lorem.ipsum/dolet/", "/dolet"),
    ("http://example.org:124/blog", "/blog"),
    ("http://example.org:124/Deep/Rab_bit/hol.e/", "/Deep/Rab_bit/hol.e"),
    # Would anybody in a sane mind actually do this?
    ("http://example.org:124/blog?lorem=ipsum&dol=et", "/blog"),
])
def test_basepath(uri: str, expected_basepath: Optional[str]) -> None:
    assert expected_basepath == auto.base_path_from_siteuri(uri)



"""Test metadata extractors."""

import os
from unittest import mock

import pytest

from nikola.metadata_extractors import (
    MetaCondition,
    check_conditions,
    default_metadata_extractors_by,
    load_defaults,
)
from nikola.plugins.compile.rest import CompileRest
from nikola.plugins.compile.markdown import CompileMarkdown
from nikola.plugins.compile.ipynb import CompileIPynb
from nikola.plugins.compile.html import CompileHtml
from nikola.post import get_meta

from .helper import FakeSite


@pytest.mark.parametrize(
    "filecount, expected, unexpected",
    [(1, "onefile", "twofile"), (2, "twofile", "onefile")],
)
@pytest.mark.parametrize(
    "format_lc, format_friendly",
    [("nikola", "Nikola"), ("toml", "TOML"), ("yaml", "YAML")],
)
def test_builtin_extractors_rest(
    metadata_extractors_by,
    testfiledir,
    filecount,
    expected,
    unexpected,
    format_lc,
    format_friendly,
):
    is_two_files = filecount == 2

    source_filename = "f-rest-{0}-{1}.rst".format(filecount, format_lc)
    metadata_filename = "f-rest-{0}-{1}.meta".format(filecount, format_lc)
    source_path = os.path.join(testfiledir, source_filename)
    metadata_path = os.path.join(testfiledir, metadata_filename)
    post = FakePost(source_path, metadata_path, {}, None, metadata_extractors_by)

    assert os.path.exists(source_path)
    if is_two_files:
        assert os.path.exists(metadata_path)

    meta, extractor = get_meta(post, None)

    assert meta
    assert extractor is metadata_extractors_by["name"][format_lc]

    assert meta["title"] == "T: reST, {0}, {1}".format(filecount, format_friendly)
    assert meta["slug"] == "s-rest-{0}-{1}".format(filecount, format_lc)
    assert expected in meta["tags"]
    assert unexpected not in meta["tags"]
    assert "meta" in meta["tags"]
    assert format_friendly in meta["tags"]
    assert "reST" in meta["tags"]
    assert meta["date"] == "2017-07-01 00:00:00 UTC"


@pytest.mark.parametrize(
    "filecount, expected, unexpected",
    [(1, "onefile", "twofile"), (2, "twofile", "onefile")],
)
def test_nikola_meta_markdown(
    metadata_extractors_by, testfiledir, filecount, expected, unexpected
):
    is_two_files = filecount == 2

    source_filename = "f-markdown-{0}-nikola.md".format(filecount)
    metadata_filename = "f-markdown-{0}-nikola.meta".format(filecount)
    source_path = os.path.join(testfiledir, source_filename)
    metadata_path = os.path.join(testfiledir, metadata_filename)
    post = FakePost(source_path, metadata_path, {}, None, metadata_extractors_by)

    assert os.path.exists(source_path)
    if is_two_files:
        assert os.path.exists(metadata_path)

    meta, extractor = get_meta(post, None)
    assert extractor is metadata_extractors_by["name"]["nikola"]

    assert meta["title"] == "T: Markdown, {0}, Nikola".format(filecount)
    assert meta["slug"] == "s-markdown-{0}-nikola".format(filecount)
    assert expected in meta["tags"]
    assert unexpected not in meta["tags"]
    assert "meta" in meta["tags"]
    assert "Nikola" in meta["tags"]
    assert "Markdown" in meta["tags"]
    assert meta["date"] == "2017-07-01 00:00:00 UTC"


@pytest.mark.parametrize(
    "compiler, fileextension, compiler_lc, name",
    [
        (CompileRest, "rst", "rest", "reST"),
        (CompileMarkdown, "md", "markdown", "Markdown"),
        (CompileIPynb, "ipynb", "ipynb", "Jupyter Notebook"),
        (CompileHtml, "html", "html", "HTML"),
    ],
)
def test_compiler_metadata(
    metadata_extractors_by, testfiledir, compiler, fileextension, compiler_lc, name
):
    source_filename = "f-{0}-1-compiler.{1}".format(compiler_lc, fileextension)
    metadata_filename = "f-{0}-1-compiler.meta".format(compiler_lc)
    title = "T: {0}, 1, compiler".format(name)
    slug = "s-{0}-1-compiler".format(compiler_lc)
    source_path = os.path.join(testfiledir, source_filename)
    metadata_path = os.path.join(testfiledir, metadata_filename)

    config = {
        "USE_REST_DOCINFO_METADATA": True,
        "MARKDOWN_EXTENSIONS": ["markdown.extensions.meta"],
    }
    site = FakeSite()
    site.config.update(config)
    compiler_obj = compiler()
    compiler_obj.set_site(site)

    post = FakePost(
        source_path, metadata_path, config, compiler_obj, metadata_extractors_by
    )

    class FakeBorg:
        current_lang = "en"

        def __call__(self):
            return self

    with mock.patch("nikola.plugins.compile." + compiler_lc + ".LocaleBorg", FakeBorg):
        meta, extractor = get_meta(post, None)

    assert meta["title"] == title
    assert meta["slug"] == slug
    assert "meta" in meta["tags"]
    assert "onefile" in meta["tags"]
    assert "compiler" in meta["tags"]
    assert name in meta["tags"]
    assert meta["date"] == "2017-07-01 00:00:00 UTC"


def test_yaml_none_handling(metadata_extractors_by):
    yaml_extractor = metadata_extractors_by["name"]["yaml"]
    meta = yaml_extractor.extract_text("---\ntitle: foo\nslug: null")
    assert meta["title"] == "foo"
    assert meta["slug"] == ""


@pytest.mark.parametrize(
    "conditions",
    [
        [(MetaCondition.config_bool, "baz"), (MetaCondition.config_present, "quux")],
        pytest.param(
            [(MetaCondition.config_bool, "quux")], marks=pytest.mark.xfail(strict=True)
        ),
        pytest.param(
            [(MetaCondition.config_present, "foobar")],
            marks=pytest.mark.xfail(strict=True),
        ),
        [(MetaCondition.extension, "bar")],
        pytest.param(
            [(MetaCondition.extension, "baz")], marks=pytest.mark.xfail(strict=True)
        ),
        [(MetaCondition.compiler, "foo")],
        pytest.param(
            [(MetaCondition.compiler, "foobar")], marks=pytest.mark.xfail(strict=True)
        ),
        pytest.param(
            [(MetaCondition.never, None), (MetaCondition.config_present, "bar")],
            marks=pytest.mark.xfail(strict=True),
        ),
    ],
)
def test_check_conditions(conditions, dummy_post):
    filename = "foo.bar"
    config = {"baz": True, "quux": False}
    assert check_conditions(dummy_post, filename, conditions, config, "")


class FakePost:
    def __init__(
        self, source_path, metadata_path, config, compiler, metadata_extractors_by
    ):
        self.source_path = source_path
        self.metadata_path = metadata_path
        self.is_two_file = True
        self.config = {"TRANSLATIONS": {"en": "./"}, "DEFAULT_LANG": "en"}
        self.config.update(config)
        self.default_lang = self.config["DEFAULT_LANG"]
        self.metadata_extractors_by = metadata_extractors_by
        if compiler:
            self.compiler = compiler

    def translated_source_path(self, _):
        return self.source_path


@pytest.fixture
def metadata_extractors_by():
    metadata_extractors = default_metadata_extractors_by()
    load_defaults(None, metadata_extractors)
    return metadata_extractors


@pytest.fixture(scope="module")
def testfiledir(test_dir):
    return os.path.join(test_dir, "data", "metadata_extractors")


@pytest.fixture(scope="module")
def dummy_post():
    class DummyCompiler:
        name = "foo"

    class DummyPost:
        compiler = DummyCompiler()

    return DummyPost()



# -*- coding: utf-8 -*-
import time

BLOG_AUTHOR = "Your Name"
BLOG_TITLE = "Demo Site"
SITE_URL = "https://example.com/"
BLOG_EMAIL = "joe@demo.site"
BLOG_DESCRIPTION = "This is a demo site for Nikola."
DEFAULT_LANG = "en"
CATEGORY_ALLOW_HIERARCHIES = False
CATEGORY_OUTPUT_FLAT_HIERARCHY = False
HIDDEN_CATEGORIES = []
HIDDEN_AUTHORS = ['Guest']
LICENSE = ""

CONTENT_FOOTER_FORMATS = {
    DEFAULT_LANG: (
        (),
        {
            "email": BLOG_EMAIL,
            "author": BLOG_AUTHOR,
            "date": time.gmtime().tm_year,
            "license": LICENSE
        }
    )
}

ADDITIONAL_METADATA = {
    "ID": "conf"
}



"""Test template-based shortcodes."""

import pytest

from nikola import Nikola


def test_mixedargs(site):
    test_template = """
arg1: {{ _args[0] }}
arg2: {{ _args[1] }}
kwarg1: {{ kwarg1 }}
kwarg2: {{ kwarg2 }}
"""

    site.shortcode_registry["test1"] = site._make_renderfunc(test_template)
    site.shortcode_registry["test2"] = site._make_renderfunc(
        "Something completely different"
    )

    res = site.apply_shortcodes("{{% test1 kwarg1=spamm arg1 kwarg2=foo,bar arg2 %}}")[
        0
    ]

    assert res.strip() == """
arg1: arg1
arg2: arg2
kwarg1: spamm
kwarg2: foo,bar""".strip()


@pytest.mark.parametrize(
    "template, data, expected_result",
    [
        # one argument
        ("arg={{ _args[0] }}", "{{% test1 onearg %}}", "arg=onearg"),
        ("arg={{ _args[0] }}", '{{% test1 "one two" %}}', "arg=one two"),
        # keyword arguments
        ("foo={{ foo }}", "{{% test1 foo=bar %}}", "foo=bar"),
        ("foo={{ foo }}", '{{% test1 foo="bar baz" %}}', "foo=bar baz"),
        ("foo={{ foo }}", '{{% test1 foo="bar baz" spamm=ham %}}', "foo=bar baz"),
        # data
        (
            "data={{ data }}",
            "{{% test1 %}}spamm spamm{{% /test1 %}}",
            "data=spamm spamm",
        ),
        ("data={{ data }}", "{{% test1 spamm %}}", "data="),
        ("data={{ data }}", "{{% test1 data=dummy %}}", "data="),
    ],
)
def test_applying_shortcode(site, template, data, expected_result):
    site.shortcode_registry["test1"] = site._make_renderfunc(template)

    assert site.apply_shortcodes(data)[0] == expected_result


@pytest.fixture(scope="module")
def site():
    s = ShortcodeFakeSite()
    s.init_plugins()
    s._template_system = None
    return s


class ShortcodeFakeSite(Nikola):
    def _get_template_system(self):
        if self._template_system is None:
            # Load template plugin
            self._template_system = self.plugin_manager.get_plugin_by_name(
                "jinja", "TemplateSystem"
            ).plugin_object
            self._template_system.set_directories(".", "cache")
            self._template_system.set_site(self)

        return self._template_system

    template_system = property(_get_template_system)



"""Check that SITE_URL with a path doesn't break links."""

import io
import os

import lxml.html
import pytest

from nikola import __main__

from .helper import cd, patch_config
from .test_demo_build import prepare_demo_site
from .test_empty_build import (  # NOQA
    test_archive_exists,
    test_avoid_double_slash_in_rss,
    test_check_files,
    test_check_links,
)


def test_relative_links(build, output_dir):
    """Check that the links in output/index.html are correct"""
    test_path = os.path.join(output_dir, "index.html")

    with io.open(test_path, "rb") as inf:
        data = inf.read()

    assert not any(
        url.startswith("..")
        for _, _, url, _ in lxml.html.fromstring(data).iterlinks()
        if url.endswith("css")
    )


def test_index_in_sitemap(build, output_dir):
    """Test that the correct path is in sitemap, and not the wrong one."""
    sitemap_path = os.path.join(output_dir, "sitemap.xml")
    with io.open(sitemap_path, "r", encoding="utf8") as inf:
        sitemap_data = inf.read()

    assert "<loc>https://example.com/</loc>" not in sitemap_data
    assert "<loc>https://example.com/foo/bar/</loc>" in sitemap_data


@pytest.fixture(scope="module")
def build(target_dir):
    """Fill the site with demo content and build it."""
    prepare_demo_site(target_dir)

    # Set the SITE_URL to have a path with subfolder
    patch_config(
        target_dir,
        (
            'SITE_URL = "https://example.com/"',
            'SITE_URL = "https://example.com/foo/bar/"',
        ),
    )

    with cd(target_dir):
        __main__.main(["build"])



"""Make sure posts only in secondary languages work."""

import os
import shutil

import pytest

import nikola.plugins.command.init
from nikola import __main__

from .helper import cd
from .test_empty_build import (  # NOQA
    test_archive_exists,
    test_avoid_double_slash_in_rss,
    test_check_files,
    test_check_links,
    test_index_in_sitemap,
)


@pytest.fixture(scope="module")
def build(target_dir, test_dir):
    """Build the site."""
    init_command = nikola.plugins.command.init.CommandInit()
    init_command.create_empty_site(target_dir)
    init_command.create_configuration(target_dir)

    src = os.path.join(test_dir, "..", "data", "translated_titles")
    for root, dirs, files in os.walk(src):
        for src_name in files:
            if src_name == "1.txt":  # English post
                continue

            rel_dir = os.path.relpath(root, src)
            dst_file = os.path.join(target_dir, rel_dir, src_name)
            src_file = os.path.join(root, src_name)
            shutil.copy2(src_file, dst_file)

    with cd(target_dir):
        __main__.main(["build"])



import datetime
import unittest.mock

import dateutil
import pytest

from nikola.nikola import LEGAL_VALUES
from nikola.utils import (
    LocaleBorg,
    LocaleBorgUninitializedException,
    TranslatableSetting,
)

TESLA_BIRTHDAY = datetime.date(1856, 7, 10)
TESLA_BIRTHDAY_DT = datetime.datetime(1856, 7, 10, 12, 34, 56)
DT_EN_US = "July 10, 1856, 12:34:56\u202fPM UTC"
DT_PL = "10 lipca 1856 12:34:56 UTC"


@pytest.mark.parametrize("initial_lang", [None, ""])
def test_initilalize_failure(initial_lang):
    with pytest.raises(ValueError):
        LocaleBorg.initialize({}, initial_lang)

    assert not LocaleBorg.initialized


@pytest.mark.parametrize("initial_lang", ["en", "pl"])
def test_initialize(initial_lang):
    LocaleBorg.initialize({}, initial_lang)
    assert LocaleBorg.initialized
    assert LocaleBorg().current_lang == initial_lang


def test_uninitialized_error():
    with pytest.raises(LocaleBorgUninitializedException):
        LocaleBorg()


@pytest.mark.parametrize(
    "locale, expected_current_lang",
    [
        ("pl", "pl"),
        pytest.param(
            "xx", "xx", id="fake language"
        ),  # used to ensure any locale can be supported
    ],
)
def test_set_locale(base_config, locale, expected_current_lang):
    LocaleBorg().set_locale(locale)
    assert LocaleBorg.initialized
    assert LocaleBorg().current_lang == expected_current_lang


def test_set_locale_for_template():
    LocaleBorg.initialize({}, "en")
    assert LocaleBorg().set_locale("xz") == ""  # empty string for template ease of use


def test_format_date_webiso_basic(base_config):
    with unittest.mock.patch("babel.dates.format_datetime") as m:
        formatted_date = LocaleBorg().formatted_date("webiso", TESLA_BIRTHDAY_DT)
        assert formatted_date == "1856-07-10T12:34:56"
        m.assert_not_called()


@pytest.mark.parametrize("lang", ["en", "pl"])
def test_format_date_basic(base_config, lang):
    LocaleBorg.initialize({}, lang)
    formatted_date = LocaleBorg().formatted_date(
        "yyyy-MM-dd HH:mm:ss", TESLA_BIRTHDAY_DT
    )
    assert formatted_date == "1856-07-10 12:34:56"


def test_format_date_long(base_config):
    assert LocaleBorg().formatted_date("long", TESLA_BIRTHDAY_DT) == DT_EN_US
    assert LocaleBorg().formatted_date("long", TESLA_BIRTHDAY_DT, "en") == DT_EN_US
    assert LocaleBorg().formatted_date("long", TESLA_BIRTHDAY_DT, "pl") == DT_PL
    LocaleBorg().set_locale("pl")
    assert LocaleBorg().formatted_date("long", TESLA_BIRTHDAY_DT) == DT_PL
    assert LocaleBorg().formatted_date("long", TESLA_BIRTHDAY_DT, "en") == DT_EN_US


def test_format_date_timezone(base_config):
    tesla_150_birthday_dtz = datetime.datetime(
        2006, 7, 10, 12, 34, 56, tzinfo=dateutil.tz.gettz("America/New_York")
    )
    formatted_date = LocaleBorg().formatted_date("long", tesla_150_birthday_dtz)
    assert formatted_date == "July 10, 2006, 12:34:56\u202fPM -0400"

    nodst = datetime.datetime(
        2006, 1, 10, 12, 34, 56, tzinfo=dateutil.tz.gettz("America/New_York")
    )
    formatted_date = LocaleBorg().formatted_date("long", nodst)
    assert formatted_date == "January 10, 2006, 12:34:56\u202fPM -0500"


@pytest.mark.parametrize(
    "english_variant, expected_date",
    [
        pytest.param("en_US", DT_EN_US, id="US"),
        pytest.param("en_GB", "10 July 1856, 12:34:56 UTC", id="GB"),
    ],
)
def test_format_date_locale_variants(english_variant, expected_date):
    LocaleBorg.initialize({"en": english_variant}, "en")
    assert LocaleBorg().formatted_date("long", TESLA_BIRTHDAY_DT, "en") == expected_date


@pytest.mark.parametrize(
    "lang, expected_string", [("en", "en July"), ("pl", "lipca pl")]
)
def test_format_date_translatablesetting(base_config, lang, expected_string):
    df = TranslatableSetting(
        "DATE_FORMAT", {"en": "'en' MMMM", "pl": "MMMM 'pl'"}, {"en": "", "pl": ""}
    )
    assert LocaleBorg().formatted_date(df, TESLA_BIRTHDAY_DT, lang) == expected_string


@pytest.mark.parametrize(
    "lang, expected_string",
    [
        pytest.param(None, "Foo July Bar", id="default"),
        pytest.param("pl", "Foo lipiec Bar", id="pl"),
    ],
)
def test_format_date_in_string_month(base_config, lang, expected_string):
    formatted_date = LocaleBorg().format_date_in_string(
        "Foo {month} Bar", TESLA_BIRTHDAY, lang
    )
    assert formatted_date == expected_string


@pytest.mark.parametrize(
    "lang, expected_string",
    [
        pytest.param(None, "Foo July 1856 Bar", id="default"),
        pytest.param("pl", "Foo lipiec 1856 Bar", id="pl"),
    ],
)
def test_format_date_in_string_month_year(base_config, lang, expected_string):
    formatted_date = LocaleBorg().format_date_in_string(
        "Foo {month_year} Bar", TESLA_BIRTHDAY, lang
    )
    assert formatted_date == expected_string


@pytest.mark.parametrize(
    "lang, expected_string",
    [
        pytest.param(None, "Foo July 10, 1856 Bar", id="default"),
        pytest.param("pl", "Foo 10 lipca 1856 Bar", id="pl"),
    ],
)
def test_format_date_in_string_month_day_year(base_config, lang, expected_string):
    formatted_date = LocaleBorg().format_date_in_string(
        "Foo {month_day_year} Bar", TESLA_BIRTHDAY, lang
    )
    assert formatted_date == expected_string


@pytest.mark.parametrize(
    "lang, expected_string",
    [
        pytest.param(None, "Foo 10 July 1856 Bar", id="default"),
        pytest.param("pl", "Foo 10 lipca 1856 Bar", id="pl"),
    ],
)
def test_format_date_in_string_month_day_year_gb(lang, expected_string):
    LocaleBorg.initialize({"en": "en_GB"}, "en")
    formatted_date = LocaleBorg().format_date_in_string(
        "Foo {month_day_year} Bar", TESLA_BIRTHDAY, lang
    )
    assert formatted_date == expected_string


@pytest.mark.parametrize(
    "message, expected_string",
    [
        ("Foo {month:'miesiąca' MMMM} Bar", "Foo miesiąca lipca Bar"),
        ("Foo {month_year:MMMM yyyy} Bar", "Foo lipca 1856 Bar"),
    ],
)
def test_format_date_in_string_customization(base_config, message, expected_string):
    formatted_date = LocaleBorg().format_date_in_string(message, TESLA_BIRTHDAY, "pl")
    assert formatted_date == expected_string


@pytest.mark.parametrize(
    "lang, expected_format",
    [("sr", "10. јул 1856. 12:34:56 UTC"), ("sr_latin", "10. jul 1856. 12:34:56 UTC")],
)
def test_locale_base(lang, expected_format):
    LocaleBorg.initialize(LEGAL_VALUES["LOCALES_BASE"], "en")
    formatted_date = LocaleBorg().formatted_date("long", TESLA_BIRTHDAY_DT, lang)
    assert formatted_date == expected_format


@pytest.fixture(autouse=True)
def localeborg_reset():
    """
    Reset the LocaleBorg before and after every test.
    """
    LocaleBorg.reset()
    assert not LocaleBorg.initialized
    try:
        yield
    finally:
        LocaleBorg.reset()
        assert not LocaleBorg.initialized


@pytest.fixture
def base_config():
    """A base config of LocaleBorg."""
    LocaleBorg.initialize({}, "en")



# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "",
    "(active)": "",
    "Also available in:": "",
    "Archive": "",
    "Atom feed": "",
    "Authors": "",
    "Categories": "",
    "Comments": "Ixfawalen",
    "LANGUAGE": "Tamaziɣt",
    "Languages:": "",
    "More posts about %s": "",
    "Newer posts": "",
    "Next post": "",
    "Next": "",
    "No posts found.": "",
    "Nothing found.": "",
    "Older posts": "",
    "Original site": "",
    "Posted:": "",
    "Posts about %s": "",
    "Posts by %s": "",
    "Posts for year %s": "",
    "Posts for {month_day_year}": "",
    "Posts for {month_year}": "",
    "Previous post": "",
    "Previous": "",
    "Publication date": "",
    "RSS feed": "",
    "Read in English": "Ɣer s tneglizt",
    "Read more": "Ɣer uggar",
    "Skip to main content": "",
    "Source": "Asagem",
    "Subcategories:": "",
    "Tags and Categories": "",
    "Tags": "",
    "Toggle navigation": "",
    "Uncategorized": "",
    "Up": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "",
    "page %d": "",
    "updated": "",
}



import os

from .helper import cd


class SomeTestError(Exception):
    """An arbitrary error to be thrown by the test."""
    pass


def test_test_helper():
    """Check that the cd test helper duly resets the directory even in spite of an error."""
    old_dir = os.getcwd()
    exception_seen = False
    try:
        with cd(".."):
            raise SomeTestError("Just raising an exception, as failing tests sometimes do.")
    except SomeTestError:
        now_dir = os.getcwd()
        assert old_dir == now_dir
        exception_seen = True
    assert exception_seen



"""
Validate links in a site which is:

* built in URL_TYPE="absolute"
* deployable to a subfolder (BASE_URL="https://example.com/foo/")
"""

import io
import os

import pytest

from nikola import __main__

from .helper import cd, patch_config
from .test_demo_build import prepare_demo_site
from .test_empty_build import (  # NOQA
    test_archive_exists,
    test_avoid_double_slash_in_rss,
    test_check_files,
    test_check_links,
)


def test_index_in_sitemap(build, output_dir):
    """
    Test that the correct path is in sitemap, and not the wrong one.

    The correct path ends in /foo/ because this is where we deploy to.
    """
    sitemap_path = os.path.join(output_dir, "sitemap.xml")
    with io.open(sitemap_path, "r", encoding="utf8") as inf:
        sitemap_data = inf.read()

    assert "<loc>https://example.com/foo/</loc>" in sitemap_data


@pytest.fixture(scope="module")
def build(target_dir):
    """Fill the site with demo content and build it."""
    prepare_demo_site(target_dir)

    patch_config(
        target_dir,
        ('SITE_URL = "https://example.com/"', 'SITE_URL = "https://example.com/foo/"'),
        ("# URL_TYPE = 'rel_path'", "URL_TYPE = 'absolute'"),
    )

    with cd(target_dir):
        __main__.main(["build"])



import dbm
import json
import subprocess
import sys


def dbm_iter(db):
    # try dictionary interface - ok in python2 and dumbdb
    try:
        return db.items()
    except Exception:
        # try firstkey/nextkey - ok for py3 dbm.gnu
        def iter_gdbm(db):
            k = db.firstkey()
            while k is not None:
                yield k, db[k]
                k = db.nextkey(k)
        return iter_gdbm(db)


def dumpdb():
    with dbm.open('.doit.db') as data:
        return {key: json.loads(value_str.decode('utf-8'))
                for key, value_str in dbm_iter(data)}


print_ = print


def print(*args, **kwargs):
    print_(*args, file=sys.stdout)
    sys.stdout.flush()


print("==> Removing stuff...")
subprocess.call(['rm', '-rf', '.doit.db', 'output', 'cache', 'cc_debug.sqlite3'])
print("==> Running first build...")
subprocess.call(['nikola', 'build'])
print("==> Fetching database...")
first = dumpdb()
print("==> Running second build...")
subprocess.call(['nikola', 'build'])
print("==> Fetching database...")
second = dumpdb()
print("==> Saving dumps...")
with open('first_dump.py', 'w', encoding='utf-8') as fh:
    fh.write(repr(first))

with open('second_dump.py', 'w', encoding='utf-8') as fh:
    fh.write(repr(second))



"""Performaning the build of an empty site."""

import io
import os

import pytest

import nikola.plugins.command.init
from nikola import __main__

from .helper import cd


def test_check_links(build, target_dir):
    with cd(target_dir):
        assert __main__.main(["check", "-l"]) is None


def test_check_files(build, target_dir):
    with cd(target_dir):
        assert __main__.main(["check", "-f"]) is None


def test_index_in_sitemap(build, output_dir):
    sitemap_path = os.path.join(output_dir, "sitemap.xml")
    with io.open(sitemap_path, "r", encoding="utf8") as inf:
        sitemap_data = inf.read()

    assert "<loc>https://example.com/</loc>" in sitemap_data


def test_avoid_double_slash_in_rss(build, output_dir):
    rss_path = os.path.join(output_dir, "rss.xml")
    with io.open(rss_path, "r", encoding="utf8") as inf:
        rss_data = inf.read()

    assert "https://example.com//" not in rss_data


def test_archive_exists(build, output_dir):
    """Ensure the build did something."""
    index_path = os.path.join(output_dir, "archive.html")
    assert os.path.isfile(index_path)


@pytest.fixture(scope="module")
def build(target_dir):
    """Build the site."""
    init_command = nikola.plugins.command.init.CommandInit()
    init_command.create_empty_site(target_dir)
    init_command.create_configuration(target_dir)

    with cd(target_dir):
        __main__.main(["build"])



"""
Test cases for Nikola ReST extensions.

A sample will be rendered by CompileRest.
One method is provided for checking the resulting HTML:

    * assert_html_contains(html, element, attributes=None, text=None)

"""

from io import StringIO

import pytest
from lxml import html as lxml_html

import nikola.plugins.compile.rest
import nikola.plugins.compile.rest.listing
from nikola.plugins.compile.rest import vimeo
from nikola.utils import _reload, LocaleBorg

from .helper import FakeSite


def test_ReST_extension():
    sample = '.. raw:: html\n\n   <iframe src="foo" height="bar">spam</iframe>'
    html = get_html_from_rst(sample)

    assert_html_contains(html, "iframe", attributes={"src": "foo"}, text="spam")

    with pytest.raises(Exception):
        assert_html_contains("eggs", {})


def test_math_extension_outputs_tex():
    """Test that math is outputting TeX code."""
    sample = r":math:`e^{ix} = \cos x + i\sin x`"
    html = get_html_from_rst(sample)

    assert_html_contains(
        html,
        "span",
        attributes={"class": "math"},
        text=r"\(e^{ix} = \cos x + i\sin x\)",
    )


def test_soundcloud_iframe():
    """Test SoundCloud iframe tag generation"""

    sample = ".. soundcloud:: SID\n   :height: 400\n   :width: 600"
    html = get_html_from_rst(sample)
    assert_html_contains(
        html,
        "iframe",
        attributes={
            "src": (
                "https://w.soundcloud.com/player/"
                "?url=http://api.soundcloud.com/"
                "tracks/SID"
            ),
            "height": "400",
            "width": "600",
        },
    )


def test_youtube_iframe():
    """Test Youtube iframe tag generation"""

    sample = ".. youtube:: YID\n   :height: 400\n   :width: 600"
    html = get_html_from_rst(sample)
    assert_html_contains(
        html,
        "iframe",
        attributes={
            "src": (
                "https://www.youtube-nocookie.com"
                "/embed/YID?rel=0&"
                "wmode=transparent"
            ),
            "height": "400",
            "width": "600",
            "frameborder": "0",
            "allowfullscreen": "",
            "allow": "encrypted-media",
        },
    )


def test_youtube_iframe_start_at():
    """Test Youtube iframe tag generation with start_at attribute"""

    sample = ".. youtube:: YID\n   :height: 400\n   :width: 600\n   :start_at: 60"
    html = get_html_from_rst(sample)
    assert_html_contains(
        html,
        "iframe",
        attributes={
            "src": (
                "https://www.youtube-nocookie.com"
                "/embed/YID?rel=0&"
                "wmode=transparent&start=60"
            ),
            "height": "400",
            "width": "600",
            "frameborder": "0",
            "allowfullscreen": "",
            "allow": "encrypted-media",
        },
    )


def test_vimeo(disable_vimeo_api_query):
    """Test Vimeo iframe tag generation"""

    sample = ".. vimeo:: VID\n   :height: 400\n   :width: 600"
    html = get_html_from_rst(sample)
    assert_html_contains(
        html,
        "iframe",
        attributes={
            "src": ("https://player.vimeo.com/" "video/VID"),
            "height": "400",
            "width": "600",
        },
    )


@pytest.mark.parametrize(
    "sample",
    [
        ".. code-block:: python\n\n   import antigravity",
        ".. sourcecode:: python\n\n   import antigravity",
    ],
)
def test_rendering_codeblock_alias(sample):
    """Test CodeBlock aliases"""
    get_html_from_rst(sample)


def test_doc_doesnt_exist():
    with pytest.raises(Exception):
        assert_html_contains("anything", {})


def test_doc():
    sample = "Sample for testing my :doc:`fake-post`"
    html = get_html_from_rst(sample)
    assert_html_contains(
        html, "a", text="Fake post", attributes={"href": "/posts/fake-post"}
    )


def test_doc_titled():
    sample = "Sample for testing my :doc:`titled post <fake-post>`"
    html = get_html_from_rst(sample)
    assert_html_contains(
        html, "a", text="titled post", attributes={"href": "/posts/fake-post"}
    )


@pytest.fixture(autouse=True, scope="module")
def localeborg_base():
    """A base config of LocaleBorg."""
    LocaleBorg.reset()
    assert not LocaleBorg.initialized
    LocaleBorg.initialize({}, "en")
    assert LocaleBorg.initialized
    assert LocaleBorg().current_lang == "en"
    try:
        yield
    finally:
        LocaleBorg.reset()
        assert not LocaleBorg.initialized


def get_html_from_rst(rst):
    """Create html output from rst string"""

    compiler = nikola.plugins.compile.rest.CompileRest()
    compiler.set_site(FakeSite())
    return compiler.compile_string(rst)[0]


class FakePost:
    def __init__(self, outfile):
        self._depfile = {outfile: []}


def assert_html_contains(html, element, attributes=None, text=None):
    """
    Test if HTML document includes an element with the given attributes
    and text content.

    The HTML is parsed with lxml for checking against the data you
    provide. The method takes an element argument, a string representing
    the *name* of an HTML tag, like "script" or "iframe".
    We will try to find this tag in the document and perform the tests
    on it. You can pass a dictionary to the attributes kwarg
    representing the name and the value of the tag attributes. The text
    kwarg takes a string argument, which will be tested against the
    contents of the HTML element.

    One last caveat: you need to url unquote your urls if you are going
    to test attributes like "src" or "link", since the HTML rendered by
    docutils will be always unquoted.
    """
    html_doc = lxml_html.parse(StringIO(html))

    try:
        tag = next(html_doc.iter(element))
    except StopIteration:
        raise Exception("<{0}> not in {1}".format(element, html))

    if attributes:
        arg_attrs = set(attributes.items())
        tag_attrs = set(tag.items())
        assert arg_attrs.issubset(tag_attrs)

    if text:
        assert text in tag.text


@pytest.fixture
def disable_vimeo_api_query():
    """
    Disable query of the vimeo api over the wire

    Set Vimeo.request_size to False for avoiding querying the Vimeo api
    over the network.
    """
    before = vimeo.Vimeo.request_size
    vimeo.Vimeo.request_size = False
    try:
        _reload(nikola.plugins.compile.rest)
        yield
    finally:
        vimeo.Vimeo.request_size = before



"""
Test that a default build of a new site based on the demo site works.

This module also is one place where green path tests (working as
expected) for the `check` command are tested.
In this case these are tested against the demo site with default
settings.
"""

import datetime
import email
import itertools
import os
import time

import feedparser
import freezegun
import pytest

import nikola.plugins.command.init
from nikola import __main__

from .helper import add_post_without_text, cd, copy_example_post
from .test_empty_build import (  # NOQA
    test_archive_exists,
    test_avoid_double_slash_in_rss,
    test_check_files,
    test_check_links,
    test_index_in_sitemap,
)

BUILDTIME = datetime.datetime(2023, 4, 5, 23, 59, 58, tzinfo=datetime.timezone.utc)


class Any:
    """Compare equal with anything. Use for expected values we don't care about."""
    def __eq__(self, _):
        return True


def rfc822(t):
    """Format a datetime according to RFC822, eg 'Wed, 05 Apr 2023 23:59:58 GMT'"""
    return email.utils.formatdate(
        time.mktime(BUILDTIME.astimezone().timetuple()),
        usegmt=True,
    )


def test_gallery_rss(build, output_dir):
    # Given a build of the demo samplesite in 'output_dir'
    # When we look for the RSS file of the "Demo" gallery
    rss_path = os.path.join(output_dir, 'galleries', 'demo', 'rss.xml')

    # Then it exists
    assert os.path.isfile(rss_path)
    # and it contains text
    with open(rss_path) as fp:
        content = fp.read()
    assert isinstance(content, str)
    assert len(content) > 0
    # and the text can be parsed as valid RSS
    parsed = feedparser.parse(content)
    # and the RSS contains top level attributes:
    assert parsed.version == 'rss20'
    # and the RSS contains feed attributes, about the gallery:
    assert parsed.feed.language == 'en'
    assert parsed.feed.link == 'https://example.com/galleries/demo/rss.xml'
    # TODO I think the following is a bug: The feed title should be the Gallery name,
    #      not the name of the gallery's final image.
    assert parsed.feed.title == 'Tesla tower1 lg'
    # TODO I think the following is a bug: The feed's subtitle (aka description) should
    #      contain the content of the gallery's index.txt.
    assert parsed.feed.subtitle == ''  # From the XML field 'description'
    assert parsed.feed.updated == rfc822(BUILDTIME)
    # and the images, as items in the RSS feed, are:
    expected_items = [
        dict(
            id='galleries/demo/tesla4_lg.jpg',
            link='https://example.com/galleries/demo/tesla4_lg.jpg',
            links=[
                Any(),
                dict(
                    href='https://example.com/galleries/demo/tesla4_lg.jpg',
                    length='30200',
                    rel='enclosure',
                    type='image/jpeg',
                ),
            ],
            published='Wed, 01 Jan 2014 00:01:00 GMT',
            title='Tesla4 lg',
        ),
        dict(
            id='galleries/demo/tesla_conducts_lg.webp',
            link='https://example.com/galleries/demo/tesla_conducts_lg.webp',
            links=[
                Any(),
                dict(
                    href='https://example.com/galleries/demo/tesla_conducts_lg.webp',
                    length='9620',
                    rel='enclosure',
                    type='image/webp',
                ),
            ],
            published='Wed, 01 Jan 2014 00:02:00 GMT',
            title='Tesla conducts lg',
        ),
        dict(
            id='galleries/demo/tesla_lightning1_lg.jpg',
            link='https://example.com/galleries/demo/tesla_lightning1_lg.jpg',
            links=[
                Any(),
                dict(
                    href='https://example.com/galleries/demo/tesla_lightning1_lg.jpg',
                    length='41123',
                    rel='enclosure',
                    type='image/jpeg',
                ),
            ],
            published='Wed, 01 Jan 2014 00:03:00 GMT',
            title='Tesla lightning1 lg',
        ),
        dict(
            id='galleries/demo/tesla_lightning2_lg.jpg',
            link='https://example.com/galleries/demo/tesla_lightning2_lg.jpg',
            links=[
                Any(),
                dict(
                    href='https://example.com/galleries/demo/tesla_lightning2_lg.jpg',
                    length='36994',
                    rel='enclosure',
                    type='image/jpeg',
                ),
            ],
            published='Wed, 01 Jan 2014 00:04:00 GMT',
            title='Tesla lightning2 lg',
        ),
        dict(
            id='galleries/demo/tesla_tower1_lg.jpg',
            link='https://example.com/galleries/demo/tesla_tower1_lg.jpg',
            links=[
                Any(),
                dict(
                    href='https://example.com/galleries/demo/tesla_tower1_lg.jpg',
                    length='18105',
                    rel='enclosure',
                    type='image/jpeg',
                )
            ],
            published='Wed, 01 Jan 2014 00:05:00 GMT',
            title='Tesla tower1 lg',
        ),
    ]
    for index, (actual, expected) in enumerate(
        itertools.zip_longest(parsed.entries, expected_items)
    ):
        for key, value in expected.items():
            assert actual[key] == value, f'item [{index}][{key!r}] {actual}'


@pytest.fixture(scope="module")
def build(target_dir):
    """Fill the site with demo content and build it."""
    prepare_demo_site(target_dir)

    with cd(target_dir):
        with freezegun.freeze_time(BUILDTIME):
            __main__.main(["build"])


def prepare_demo_site(target_dir):
    init_command = nikola.plugins.command.init.CommandInit()
    init_command.copy_sample_site(target_dir)
    init_command.create_configuration(target_dir)

    posts_dir = os.path.join(target_dir, "posts")
    copy_example_post(posts_dir)
    add_post_without_text(posts_dir)



# -*- coding: utf-8 -*-

# Copyright © 2012-2024 Chris Warrick and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import nikola.plugin_manager

from .helper import FakeSite
from nikola.plugin_manager import PluginManager
from pathlib import Path


def test_locate_plugins_finds_core_plugins():
    """Ensure that locate_plugins can find some core plugins."""
    places = [Path(nikola.plugin_manager.__file__).parent / "plugins"]
    plugin_manager = PluginManager(places)
    candidates = plugin_manager.locate_plugins()
    plugin_names = [p.name for p in candidates]
    assert plugin_manager.candidates == candidates

    assert "emoji" in plugin_names
    assert "copy_assets" in plugin_names
    assert "scan_posts" in plugin_names

    template_plugins = [p for p in candidates if p.category == "TemplateSystem"]
    template_plugins.sort(key=lambda p: p.name)
    assert len(template_plugins) == 2
    assert template_plugins[0].name == "jinja"
    assert template_plugins[1].name == "mako"


def test_locate_plugins_finds_core_and_custom_plugins():
    """Ensure that locate_plugins can find some custom plugins."""
    places = [
        Path(nikola.plugin_manager.__file__).parent / "plugins",
        Path(__file__).parent / "data" / "plugin_manager",
    ]
    plugin_manager = PluginManager(places)
    candidates = plugin_manager.locate_plugins()
    plugin_names = [p.name for p in candidates]
    assert plugin_manager.candidates == candidates

    assert "emoji" in plugin_names
    assert "copy_assets" in plugin_names
    assert "scan_posts" in plugin_names

    assert "first" in plugin_names
    assert "2nd" in plugin_names

    first_plugin = next(p for p in candidates if p.name == "first")
    second_plugin = next(p for p in candidates if p.name == "2nd")

    assert first_plugin.category == "Command"
    assert first_plugin.compiler == "foo"
    assert first_plugin.source_dir == places[1]

    assert second_plugin.category == "ConfigPlugin"
    assert second_plugin.compiler is None
    assert second_plugin.source_dir == places[1] / "second"


def test_load_plugins():
    """Ensure that locate_plugins can load some core and custom plugins."""
    places = [
        Path(nikola.plugin_manager.__file__).parent / "plugins",
        Path(__file__).parent / "data" / "plugin_manager",
    ]
    plugin_manager = PluginManager(places)
    candidates = plugin_manager.locate_plugins()
    plugins_to_load = [p for p in candidates if p.name in {"first", "2nd", "emoji"}]

    plugin_manager.load_plugins(plugins_to_load)

    assert len(plugin_manager.plugins) == 3
    assert plugin_manager._plugins_by_category["ShortcodePlugin"][0].name == "emoji"
    assert plugin_manager._plugins_by_category["Command"][0].name == "first"
    assert plugin_manager._plugins_by_category["ConfigPlugin"][0].name == "2nd"

    site = FakeSite()
    for plugin in plugin_manager.plugins:
        plugin.plugin_object.set_site(site)

    assert "emoji" in site.shortcode_registry
    assert plugin_manager.get_plugin_by_name("first", "Command").plugin_object.one_site_set
    assert plugin_manager.get_plugin_by_name("2nd").plugin_object.two_site_set
    assert plugin_manager.get_plugin_by_name("2nd", "Command") is None


def test_load_plugins_twice():
    """Ensure that extra plugins can be added."""
    places = [
        Path(nikola.plugin_manager.__file__).parent / "plugins",
        Path(__file__).parent / "data" / "plugin_manager",
    ]
    plugin_manager = PluginManager(places)
    candidates = plugin_manager.locate_plugins()
    plugins_to_load_first = [p for p in candidates if p.name in {"first", "emoji"}]
    plugins_to_load_second = [p for p in candidates if p.name in {"2nd"}]

    plugin_manager.load_plugins(plugins_to_load_first)
    assert len(plugin_manager.plugins) == 2
    plugin_manager.load_plugins(plugins_to_load_second)
    assert len(plugin_manager.plugins) == 3


def test_load_plugins_skip_mismatching_category(caplog):
    """If a plugin specifies a different category than it actually implements, refuse to load it."""
    places = [
        Path(__file__).parent / "data" / "plugin_manager",
    ]
    plugin_manager = PluginManager(places)
    candidates = plugin_manager.locate_plugins()
    plugins_to_load = [p for p in candidates if p.name in {"broken"}]
    plugin_to_load = plugins_to_load[0]
    assert len(plugins_to_load) == 1

    plugin_manager.load_plugins(plugins_to_load)

    py_file = plugin_to_load.source_dir / "broken.py"
    assert f"{plugin_to_load.plugin_id} ({py_file}) has category '{plugin_to_load.category}' in the .plugin file, but the implementation class <class 'tests.data.plugin_manager.broken.BrokenPlugin'> does not inherit from this category - plugin will not be loaded" in caplog.text
    assert len(plugin_manager.plugins) == 0



"""Test if PAGE_INDEX works, with different PRETTY_URLS=False settings."""

import io
import os

import pytest

import nikola.plugins.command.init
from nikola import __main__
from nikola.utils import makedirs

from .helper import append_config, cd
from .test_empty_build import (  # NOQA
    test_archive_exists,
    test_avoid_double_slash_in_rss,
    test_check_files,
    test_check_links,
    test_index_in_sitemap,
)


def get_last_folder_as_id(value):
    """Use the last part of the directories as test identifier."""
    if isinstance(value, (tuple,)):
        return value[-1]

    return value


@pytest.mark.parametrize(
    "dirs, expected_file",
    [
        (("pages",), "page0"),
        (("pages", "subdir1"), "page1"),
        (("pages", "subdir1"), "page2"),
        (("pages", "subdir2"), "page3"),
        (("pages", "subdir3"), "page4"),
    ],
    ids=get_last_folder_as_id,
)
def test_page_index(build, output_dir, dirs, expected_file, output_path_func):
    """Test PAGE_INDEX - Do all files exist?"""
    path_func = output_path_func

    checkdir = os.path.join(output_dir, *dirs)

    assert os.path.isfile(path_func(checkdir, expected_file))


@pytest.mark.parametrize(
    "dirs, expected_index_file",
    [
        (("pages",), "index.html"),
        (("pages", "subdir1"), "index.html"),
        (("pages", "subdir2"), "index.html"),
        (("pages", "subdir3"), "index.php"),
    ],
    ids=get_last_folder_as_id,
)
def test_page_index_in_subdir(build, output_dir, dirs, expected_index_file):
    """Test PAGE_INDEX - Do index files in subdir exist?"""
    checkdir = os.path.join(output_dir, *dirs)

    assert os.path.isfile(os.path.join(checkdir, expected_index_file))
    if expected_index_file == "index.php":
        assert not os.path.isfile(os.path.join(checkdir, "index.html"))


@pytest.fixture(scope="module")
def output_path_func():
    def output_path(dir, name):
        """Make a file path to the output."""
        return os.path.join(dir, name + ".html")

    return output_path


def test_page_index_content_in_pages(build, output_dir):
    """Do the indexes only contain the pages the should?"""
    pages = os.path.join(output_dir, "pages")

    with io.open(os.path.join(pages, "index.html"), "r", encoding="utf-8") as fh:
        pages_index = fh.read()

    assert "Page 0" in pages_index
    assert "Page 1" not in pages_index
    assert "Page 2" not in pages_index
    assert "Page 3" not in pages_index
    assert "Page 4" not in pages_index
    assert "This is not the page index" not in pages_index


def test_page_index_content_in_subdir1(build, output_dir):
    """Do the indexes only contain the pages the should?"""
    subdir1 = os.path.join(output_dir, "pages", "subdir1")

    with io.open(os.path.join(subdir1, "index.html"), "r", encoding="utf-8") as fh:
        subdir1_index = fh.read()

    assert "Page 0" not in subdir1_index
    assert "Page 1" in subdir1_index
    assert "Page 2" in subdir1_index
    assert "Page 3" not in subdir1_index
    assert "Page 4" not in subdir1_index
    assert "This is not the page index" not in subdir1_index


def test_page_index_content_in_subdir2(build, output_dir):
    """Do the indexes only contain the pages the should?"""
    subdir2 = os.path.join(output_dir, "pages", "subdir2")

    with io.open(os.path.join(subdir2, "index.html"), "r", encoding="utf-8") as fh:
        subdir2_index = fh.read()

    assert "Page 0" not in subdir2_index
    assert "Page 1" not in subdir2_index
    assert "Page 2" not in subdir2_index
    assert "Page 3" not in subdir2_index
    assert "Page 4" not in subdir2_index
    assert "This is not the page index." in subdir2_index


def test_page_index_content_in_subdir3(build, output_dir):
    """Do the indexes only contain the pages the should?"""
    subdir3 = os.path.join(output_dir, "pages", "subdir3")

    with io.open(os.path.join(subdir3, "index.php"), "r", encoding="utf-8") as fh:
        subdir3_index = fh.read()

    assert "Page 0" not in subdir3_index
    assert "Page 1" not in subdir3_index
    assert "Page 2" not in subdir3_index
    assert "Page 3" not in subdir3_index
    assert "Page 4" not in subdir3_index
    assert "This is not the page index either." in subdir3_index


@pytest.fixture(scope="module")
def build(target_dir):
    """Build the site."""
    init_command = nikola.plugins.command.init.CommandInit()
    init_command.create_empty_site(target_dir)
    init_command.create_configuration(target_dir)

    create_pages(target_dir)

    append_config(
        target_dir,
        """
PAGE_INDEX = True
PRETTY_URLS = False
PAGES = PAGES + (('pages/*.php', 'pages', 'page.tmpl'),)
""",
    )

    with cd(target_dir):
        __main__.main(["build"])


def create_pages(target_dir):
    pages = os.path.join(target_dir, "pages")
    subdir1 = os.path.join(target_dir, "pages", "subdir1")
    subdir2 = os.path.join(target_dir, "pages", "subdir2")
    subdir3 = os.path.join(target_dir, "pages", "subdir3")

    makedirs(subdir1)
    makedirs(subdir2)
    makedirs(subdir3)

    with io.open(os.path.join(pages, "page0.txt"), "w+", encoding="utf8") as outf:
        outf.write(
            """\
.. title: Page 0
.. slug: page0

This is page 0.
"""
        )

    with io.open(os.path.join(subdir1, "page1.txt"), "w+", encoding="utf8") as outf:
        outf.write(
            """\
.. title: Page 1
.. slug: page1

This is page 1.
"""
        )

    with io.open(os.path.join(subdir1, "page2.txt"), "w+", encoding="utf8") as outf:
        outf.write(
            """\
.. title: Page 2
.. slug: page2

This is page 2.
"""
        )

    with io.open(os.path.join(subdir2, "page3.txt"), "w+", encoding="utf8") as outf:
        outf.write(
            """\
.. title: Page 3
.. slug: page3

This is page 3.
"""
        )

    with io.open(os.path.join(subdir2, "foo.txt"), "w+", encoding="utf8") as outf:
        outf.write(
            """\
.. title: Not the page index
.. slug: index

This is not the page index.
"""
        )

    with io.open(os.path.join(subdir3, "page4.txt"), "w+", encoding="utf8") as outf:
        outf.write(
            """\
.. title: Page 4
.. slug: page4

This is page 4.
"""
        )

    with io.open(os.path.join(subdir3, "bar.php"), "w+", encoding="utf8") as outf:
        outf.write(
            """\
.. title: Still not the page index
.. slug: index

This is not the page index either.
"""
        )



import sphinx.parsers
from docutils.statemachine import StringList
from typing import TYPE_CHECKING, Any, Dict, List, Type, Union

if TYPE_CHECKING:
    from sphinx.application import Sphinx

TITLE_MARKER = ".. title:"


class NikolaTitlesRSTParser(sphinx.parsers.RSTParser):
    def decorate(self, content: StringList) -> None:
        """Preprocess reST content before parsing."""
        super().decorate(content)
        for line in content[:20]:
            if line.startswith(TITLE_MARKER):
                title = line[len(TITLE_MARKER) :].strip()
                fence = "=" * len(title)
                content.insert(0, "", "<generated>", 0)
                content.insert(0, fence, "<generated>", 0)
                content.insert(0, title, "<generated>", 0)
                content.insert(0, fence, "<generated>", 0)


def setup(app: "Sphinx") -> Dict[str, Any]:
    app.add_source_parser(NikolaTitlesRSTParser, override=True)

    return {
        "version": "0.1.0",
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



import os
from unittest import mock

import pytest

import nikola.plugins.command.import_wordpress


def test_create_import_work_without_argument(import_command):
    """
    Running import command without an argument must not fail.
    It should show the proper usage of the command.
    """
    import_command.execute()


@pytest.mark.parametrize(
    "key, expected_value",
    [
        ("DEFAULT_LANG", "de"),
        ("BLOG_TITLE", "Wordpress blog title"),
        ("BLOG_DESCRIPTION", "Nikola test blog ;) - with moré Ümläüts"),
        ("SITE_URL", "http://some.blog/"),
        ("BLOG_EMAIL", "mail@some.blog"),
        ("BLOG_AUTHOR", "Niko"),
    ],
)
def test_populate_context(import_command, import_filename, key, expected_value):
    channel = import_command.get_channel_from_file(import_filename)
    import_command.html2text = False
    import_command.transform_to_markdown = False
    import_command.transform_to_html = False
    import_command.use_wordpress_compiler = False
    import_command.translations_pattern = "{path}.{lang}.{ext}"
    context = import_command.populate_context(channel)

    for required_key in ("POSTS", "PAGES", "COMPILERS"):
        assert required_key in context

    assert expected_value == context[key]


def test_importing_posts_and_attachments(module, import_command, import_filename):
    channel = import_command.get_channel_from_file(import_filename)
    import_command.base_dir = ""
    import_command.output_folder = "new_site"
    import_command.squash_newlines = True
    import_command.no_downloads = False
    import_command.export_categories_as_categories = False
    import_command.export_comments = False
    import_command.html2text = False
    import_command.transform_to_markdown = False
    import_command.transform_to_html = False
    import_command.use_wordpress_compiler = False
    import_command.tag_saniziting_strategy = "first"
    import_command.separate_qtranslate_content = False
    import_command.translations_pattern = "{path}.{lang}.{ext}"

    import_command.context = import_command.populate_context(channel)

    # Ensuring clean results
    import_command.url_map = {}
    module.links.clear()

    write_metadata = mock.MagicMock()
    write_content = mock.MagicMock()
    write_attachments_info = mock.MagicMock()
    download_mock = mock.MagicMock()

    with mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.write_content",
        write_content,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.write_metadata",
        write_metadata,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.download_url_content_to_file",
        download_mock,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.write_attachments_info",
        write_attachments_info,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.os.makedirs"
    ):
        import_command.import_posts(channel)

    assert download_mock.called
    qpath = "new_site/files/wp-content/uploads/2008/07/arzt_und_pfusch-sick-cover.png"
    download_mock.assert_any_call(
        "http://some.blog/wp-content/uploads/2008/07/arzt_und_pfusch-sick-cover.png",
        qpath.replace("/", os.sep),
    )

    assert write_metadata.called
    write_metadata.assert_any_call(
        "new_site/pages/kontakt.meta".replace("/", os.sep),
        "Kontakt",
        "kontakt",
        "2009-07-16 20:20:32",
        "",
        [],
        **{"wp-status": "publish"}
    )

    assert write_content.called
    write_content.assert_any_call(
        "new_site/posts/2007/04/hoert.md".replace("/", os.sep),
        """An image.

<img class="size-full wp-image-16" title="caption test" src="http://some.blog/wp-content/uploads/2009/07/caption_test.jpg" alt="caption test" width="739" height="517" />

Some source code.

```Python

import sys
print sys.version

```

The end.
""",
        True,
    )

    assert write_attachments_info.called
    write_attachments_info.assert_any_call(
        "new_site/posts/2008/07/arzt-und-pfusch-s-i-c-k.attachments.json".replace(
            "/", os.sep
        ),
        {
            10: {
                "wordpress_user_name": "Niko",
                "files_meta": [
                    {"width": 300, "height": 299},
                    {"width": 150, "size": "thumbnail", "height": 150},
                ],
                "excerpt": "Arzt+Pfusch - S.I.C.K.",
                "date_utc": "2009-07-16 19:40:37",
                "content": "Das Cover von Arzt+Pfusch - S.I.C.K.",
                "files": [
                    "/wp-content/uploads/2008/07/arzt_und_pfusch-sick-cover.png",
                    "/wp-content/uploads/2008/07/arzt_und_pfusch-sick-cover-150x150.png",
                ],
                "title": "Arzt+Pfusch - S.I.C.K.",
            }
        },
    )

    write_content.assert_any_call(
        "new_site/posts/2008/07/arzt-und-pfusch-s-i-c-k.md".replace("/", os.sep),
        """<img class="size-full wp-image-10 alignright" title="Arzt+Pfusch - S.I.C.K." src="http://some.blog/wp-content/uploads/2008/07/arzt_und_pfusch-sick-cover.png" alt="Arzt+Pfusch - S.I.C.K." width="210" height="209" />Arzt+Pfusch - S.I.C.K.Gerade bin ich \xfcber das Album <em>S.I.C.K</em> von <a title="Arzt+Pfusch" href="http://www.arztpfusch.com/" target="_blank">Arzt+Pfusch</a> gestolpert, welches Arzt+Pfusch zum Download f\xfcr lau anbieten. Das Album steht unter einer Creative Commons <a href="http://creativecommons.org/licenses/by-nc-nd/3.0/de/">BY-NC-ND</a>-Lizenz.
Die Ladung <em>noisebmstupidevildustrial</em> gibts als MP3s mit <a href="http://www.archive.org/download/dmp005/dmp005_64kb_mp3.zip">64kbps</a> und <a href="http://www.archive.org/download/dmp005/dmp005_vbr_mp3.zip">VBR</a>, als Ogg Vorbis und als FLAC (letztere <a href="http://www.archive.org/details/dmp005">hier</a>). <a href="http://www.archive.org/download/dmp005/dmp005-artwork.zip">Artwork</a> und <a href="http://www.archive.org/download/dmp005/dmp005-lyrics.txt">Lyrics</a> gibts nochmal einzeln zum Download.""",
        True,
    )
    write_content.assert_any_call(
        "new_site/pages/kontakt.md".replace("/", os.sep),
        """<h1>Datenschutz</h1>
Ich erhebe und speichere automatisch in meine Server Log Files Informationen, die dein Browser an mich \xfcbermittelt. Dies sind:
<ul>
    <li>Browsertyp und -version</li>
    <li>verwendetes Betriebssystem</li>
    <li>Referrer URL (die zuvor besuchte Seite)</li>
    <li>IP Adresse des zugreifenden Rechners</li>
    <li>Uhrzeit der Serveranfrage.</li>
</ul>
Diese Daten sind f\xfcr mich nicht bestimmten Personen zuordenbar. Eine Zusammenf\xfchrung dieser Daten mit anderen Datenquellen wird nicht vorgenommen, die Daten werden einzig zu statistischen Zwecken erhoben.""",
        True,
    )

    assert len(import_command.url_map) > 0

    assert (
        "http://some.blog/posts/2007/04/hoert.html" ==
        import_command.url_map["http://some.blog/2007/04/hoert/"]
    )
    assert (
        "http://some.blog/posts/2008/07/arzt-und-pfusch-s-i-c-k.html" ==
        import_command.url_map["http://some.blog/2008/07/arzt-und-pfusch-s-i-c-k/"]
    )
    assert (
        "http://some.blog/pages/kontakt.html" ==
        import_command.url_map["http://some.blog/kontakt/"]
    )

    image_thumbnails = [
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-64x64.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-300x175.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-36x36.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-24x24.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-96x96.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-96x96.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-48x48.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-96x96.png",
        "http://some.blog/wp-content/uploads/2012/12/2012-12-19-1355925145_1024x600_scrot-150x150.png",
    ]

    for link in image_thumbnails:
        assert link in module.links


def test_transforming_content(import_command):
    """Applying markup conversions to content."""

    import_command.html2text = False
    import_command.transform_to_markdown = False
    import_command.transform_to_html = False
    import_command.use_wordpress_compiler = False
    import_command.translations_pattern = "{path}.{lang}.{ext}"

    transform_code = mock.MagicMock()
    transform_caption = mock.MagicMock()
    transform_newlines = mock.MagicMock()

    with mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.transform_code",
        transform_code,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.transform_caption",
        transform_caption,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.transform_multiple_newlines",
        transform_newlines,
    ):
        import_command.transform_content("random content", "wp", None)

    assert transform_code.called
    assert transform_caption.called
    assert transform_newlines.called


def test_transforming_source_code(import_command):
    """
    Tests the handling of sourcecode tags.
    """
    content = """Hello World.
[sourcecode language="Python"]
import sys
print sys.version
[/sourcecode]"""

    content = import_command.transform_code(content)

    assert "[/sourcecode]" not in content
    assert "[sourcecode language=" not in content

    replaced_content = """Hello World.
```Python

import sys
print sys.version

```"""
    assert content == replaced_content


def test_transform_caption(import_command):
    caption = '[caption id="attachment_16" align="alignnone" width="739" caption="beautiful picture"]<img class="size-full wp-image-16" src="http://some.blog/wp-content/uploads/2009/07/caption_test.jpg" alt="beautiful picture" width="739" height="517" />[/caption]'
    transformed_content = import_command.transform_caption(caption)

    expected_content = '<img class="size-full wp-image-16" src="http://some.blog/wp-content/uploads/2009/07/caption_test.jpg" alt="beautiful picture" width="739" height="517" />'

    assert transformed_content == expected_content


def test_transform_multiple_captions_in_a_post(import_command):
    content = """asdasdas
[caption id="attachment_16" align="alignnone" width="739" caption="beautiful picture"]<img class="size-full wp-image-16" src="http://some.blog/wp-content/uploads/2009/07/caption_test.jpg" alt="beautiful picture" width="739" height="517" />[/caption]
asdasdas
asdasdas
[caption id="attachment_16" align="alignnone" width="739" caption="beautiful picture"]<img class="size-full wp-image-16" title="pretty" src="http://some.blog/wp-content/uploads/2009/07/caption_test.jpg" alt="beautiful picture" width="739" height="517" />[/caption]
asdasdas"""

    expected_content = """asdasdas
<img class="size-full wp-image-16" src="http://some.blog/wp-content/uploads/2009/07/caption_test.jpg" alt="beautiful picture" width="739" height="517" />
asdasdas
asdasdas
<img class="size-full wp-image-16" title="pretty" src="http://some.blog/wp-content/uploads/2009/07/caption_test.jpg" alt="beautiful picture" width="739" height="517" />
asdasdas"""

    assert expected_content == import_command.transform_caption(content)


def test_transform_multiple_newlines(import_command):
    content = """This


has



way to many

newlines.


"""
    expected_content = """This

has

way to many

newlines.

"""
    import_command.squash_newlines = False
    assert content == import_command.transform_multiple_newlines(content)

    import_command.squash_newlines = True
    assert expected_content == import_command.transform_multiple_newlines(content)


def test_transform_caption_with_link_inside(import_command):
    content = """[caption caption="Fehlermeldung"]<a href="http://some.blog/openttd-missing_sound.png"><img class="size-thumbnail wp-image-551" title="openttd-missing_sound" src="http://some.blog/openttd-missing_sound-150x150.png" alt="Fehlermeldung" /></a>[/caption]"""
    transformed_content = import_command.transform_caption(content)

    expected_content = """<a href="http://some.blog/openttd-missing_sound.png"><img class="size-thumbnail wp-image-551" title="openttd-missing_sound" src="http://some.blog/openttd-missing_sound-150x150.png" alt="Fehlermeldung" /></a>"""
    assert expected_content == transformed_content


def test_get_configuration_output_path(import_command):
    import_command.output_folder = "new_site"
    default_config_path = os.path.join("new_site", "conf.py")

    import_command.import_into_existing_site = False
    assert default_config_path == import_command.get_configuration_output_path()

    import_command.import_into_existing_site = True
    config_path_with_timestamp = import_command.get_configuration_output_path()

    assert default_config_path != config_path_with_timestamp
    assert import_command.name in config_path_with_timestamp


def test_write_content_does_not_detroy_text(import_command):
    content = b"""FOO"""
    open_mock = mock.mock_open()
    with mock.patch("nikola.plugins.basic_import.open", open_mock, create=True):
        import_command.write_content("some_file", content)

    open_mock.assert_has_calls(
        [
            mock.call(u"some_file", u"wb+"),
            mock.call().__enter__(),
            mock.call().write(b"<html><body><p>FOO</p></body></html>"),
            mock.call().__exit__(None, None, None),
        ]
    )


def test_configure_redirections(import_command):
    """
    Testing the configuration of the redirections.

    We need to make sure that we have valid sources and target links.
    """
    url_map = {"/somewhere/else": "http://foo.bar/posts/somewhereelse.html"}

    redirections = import_command.configure_redirections(url_map)

    assert 1 == len(redirections)
    assert ("somewhere/else/index.html", "/posts/somewhereelse.html") in redirections


@pytest.mark.parametrize(
    "options, additional_args",
    [
        pytest.param(None, None, id="only import filename"),
        ({"output_folder": "some_folder"}, None),
        (None, ["folder_argument"]),
    ],
)
def test_create_import(
    patched_import_command, import_filename, mocks, options, additional_args
):
    arguments = {"args": [import_filename]}
    if options:
        arguments["options"] = options
    if additional_args:
        arguments["args"].extend(additional_args)

    patched_import_command.execute(**arguments)

    for applied_mock in mocks:
        assert applied_mock.called

    assert patched_import_command.exclude_drafts is False


@pytest.mark.parametrize(
    "options",
    [
        {"exclude_drafts": True},
        {"exclude_drafts": True, "output_folder": "some_folder"},
    ],
)
def test_ignoring_drafts_during_import(
    patched_import_command, import_filename, options
):
    arguments = {"options": options, "args": [import_filename]}

    patched_import_command.execute(**arguments)
    assert patched_import_command.exclude_drafts is True


@pytest.fixture
def import_command(module):
    command = module.CommandImportWordpress()
    command.onefile = False
    return command


@pytest.fixture
def module():
    return nikola.plugins.command.import_wordpress


@pytest.fixture
def import_filename(test_dir):
    return os.path.abspath(
        os.path.join(
            test_dir, "data", "wordpress_import", "wordpress_export_example.xml"
        )
    )


@pytest.fixture
def patched_import_command(import_command, testsite, mocks):
    """
    Import command with disabled site generation and various functions mocked.
    """
    data_import, site_generation, write_urlmap, write_configuration = mocks

    import_command.site = testsite
    with mock.patch("os.system", site_generation), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.import_posts",
        data_import,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.write_urlmap_csv",
        write_urlmap,
    ), mock.patch(
        "nikola.plugins.command.import_wordpress.CommandImportWordpress.write_configuration",
        write_configuration,
    ):
        yield import_command


@pytest.fixture
def testsite():
    return FakeSite()


class FakeSite:
    def link(self, *args, **kwargs):
        # We need a link function.
        # Stubbed because there is nothing done with the results.
        pass


@pytest.fixture
def mocks():
    "Mocks to be used in `patched_import_command`"
    return [
        mock.MagicMock(name="data_import"),
        mock.MagicMock(name="site_generation"),
        mock.MagicMock(name="write_urlmap"),
        mock.MagicMock(name="write_configuration"),
    ]
